# -*- coding: utf-8 -*-

"""
frojd_fabric.api
----------------
This module implements the frojd_fabric api.
"""

import time
from fabric.api import run
from fabric.state import env
from fabric.decorators import task, runs_once
from fabric.context_managers import cd
from fabric.contrib.files import exists
from utils import run_task
from .logger import logger
import paths
from hooks import run_hook, has_hook


def report(msg, err=None):
    logger.error(msg)

    if err:
        logger.error(err)

    if hasattr(env, "raise_errors") and env.raise_errors:
        raise Exception(msg)


@runs_once
def init_tasks():
    """
    Performs basic setup before any of the tasks are run. All tasks needs to
    run this before continuing. It only fires once.
    """

    # Make sure exist are set
    if "exists" not in env:
        env.exists = exists

    if "run" not in env:
        env.run = run

    if "cd" not in env:
        env.cd = cd

    if "max_releases" not in env:
        env.max_releases = 5

    run_hook("init_tasks")


@task
def setup():
    """
    Creates shared and upload directory then fires setup to recipes.
    """

    init_tasks()

    run_hook("before_setup")

    # Create shared folder
    env.run("mkdir -p %s" % (paths.get_shared_path()))
    env.run("chmod 755 %s" % (paths.get_shared_path()))

    # Create backup folder
    env.run("mkdir -p %s" % (paths.get_backup_path()))
    env.run("chmod 750 %s" % (paths.get_backup_path()))

    # Create uploads folder
    env.run("mkdir -p %s" % (paths.get_upload_path()))
    env.run("chmod 775 %s" % (paths.get_upload_path()))

    run_hook("setup")
    run_hook("after_setup")


@task
def deploy():
    """
    Performs a deploy by invoking copy, then generating next release name and
    invoking necessary hooks.
    """

    init_tasks()

    if not has_hook("copy"):
        return report("No copy method has been defined")

    if not env.exists(paths.get_shared_path()):
        return report("You need to run setup before running deploy")

    run_hook("before_deploy")

    release_name = int(time.time()*1000)
    release_path = paths.get_releases_path(release_name)

    env.current_release = release_path

    try:
        run_hook("copy")
    except Exception, e:
        return report("Error occurred on copy. Aborting deploy", err=e)

    # Symlink current folder
    if not env.exists(paths.get_source_path(release_name)):
        return report("Source path not found '%s'" %
                      paths.get_source_path(release_name))

    paths.symlink(paths.get_source_path(release_name),
                  paths.get_current_path())

    try:
        run_hook("deploy")
    except Exception, e:
        message = "Error occurred on deploy, starting rollback..."

        logger.error(message)
        logger.error(e)

        run_task("rollback")
        return report("Error occurred on deploy")

    # Clean older releases
    if "max_releases" in env:
        cleanup_releases(int(env.max_releases))

    run_hook("after_deploy")

    logger.info("Deploy complete")


@task
def rollback():
    """
    Rolls back to previous release
    """

    init_tasks()

    run_hook("before_rollback")

    # Remove current version
    current_release = paths.get_current_release_path()
    if current_release:
        env.run("rm -rf %s" % current_release)

    # Restore previous version
    old_release = paths.get_current_release_name()
    if old_release:
        paths.symlink(paths.get_source_path(old_release),
                      paths.get_current_path())

    run_hook("rollback")
    run_hook("after_rollback")

    logger.info("Rollback complete")


@task
def cleanup_releases(limit=5):
    """
    Removes older releases.
    """

    init_tasks()

    max_versions = limit + 1

    env.run("ls -dt %s/*/ | tail -n +%s | xargs rm -rf" % (
        paths.get_releases_path(),
        max_versions)
    )


@task
def debug():
    """
    Outputs debug information, needs to run before the task".

    Example:
        fab prod debug deploy.
    """

    from fabric.network import ssh

    init_tasks()
    ssh.util.log_to_file("frojd_fabric-debug.log", 10)


@task
def test():
    """
    Prints environment information.
    """

    init_tasks()

    # TODO: Added local test support
    env.run("cat /etc/*-release")  # List linux dist info



# -*- coding: utf-8 -*-

"""
frojd_fabric.ext.mysql
----------------------
Mysql extension that handles backup and restore (future).

Params:
    env.mysql_user
    env.mysql_password
    env.mysql_host
    env.mysql_db
"""

from fabric.decorators import task
from fabric.state import env
from frojd_fabric import paths


@task
def backup_db(release=None, limit=5):
    """
    Backup database and associate it with current release
    """

    if not release:
        release = paths.get_current_release_name()

    max_versions = limit+1

    if not release:
        return

    env.run("mkdir -p %s" % paths.get_backup_path("mysql"))

    backup_file = "mysql/%s.sql.gz" % release
    backup_path = paths.get_backup_path(backup_file)

    env.run("mysqldump -u %s -p%s -h %s %s | gzip -c > %s" %
            (env.mysql_user, env.mysql_password, env.mysql_host, env.mysql_db,
             backup_path))

    # Remove older releases
    env.run("ls -dt %s/* | tail -n +%s | xargs rm -rf" % (
        paths.get_backup_path("mysql"),
        max_versions)
    )


@task
def restore_db(release=None):
    """
    Restores backup back to version, uses current version by default.
    """

    if not release:
        release = paths.get_current_release_name()

    if not release:
        raise Exception("Release %s was not found" % release)

    backup_file = "mysql/%s.sql.gz" % release
    backup_path = paths.get_backup_path(backup_file)

    if not env.exists(backup_path):
        raise Exception("Backup file %s not found" % backup_path)

    env.run("gunzip < %s | mysql -u %s -p%s -h %s %s" %
            (backup_path, env.mysql_user, env.mysql_password, env.mysql_host,
             env.mysql_db))



# -*- coding: utf-8 -*-

"""
frojd_fabric.logger
-------------------
Root logger for library.
"""

import logging
import sys


logger = logging.getLogger(__name__)

out_hdlr = logging.StreamHandler(sys.stdout)
out_hdlr.setFormatter(logging.Formatter('%(asctime)s [%(levelname)s] %(name)s: %(message)s'))
out_hdlr.setLevel(logging.INFO)

logger.addHandler(out_hdlr)
logger.setLevel(logging.INFO)




# -*- coding: utf-8 -*-

"""
frojd_fabric.ext.virtualenv
---------------------------
Holds methods for managing virtualenv.
"""

from unipath import Path
from fabric.decorators import task
from fabric.operations import prompt
from fabric.state import env

from frojd_fabric import paths
from ..logger import logger


@task
def create_venv():
    if "venv_path" not in env:
        raise Exception("No env.venv_path has been specified")

    # Checks if venv exist and prompts user if reinstalling is an option
    if env.exists(Path(get_path(), "bin")):
        logger.warn("Virtualenv is already installed")
        prompt_result = prompt("Install anyways:", default="no")

        if prompt_result == "no":
            return

    env.run("virtualenv %s" % get_path())


@task
def update_requirements():
    if "requirements" not in env:
        raise Exception("Missing env.requirements")

    env.run("pip install -r %s" % Path(
        env.current_release, "requirements", env.requirements),
    )


def get_path():
    if "venv_path" in env:
        return env.venv_path

    return paths.get_deploy_path("venv")





# -*- coding: utf-8 -*-

"""
frojd_fabric.recipes.wordpress
-----------------------------
This is a basic wordpress recipe that handles shared htaccess, config and files
"""

from fabric.state import env
from frojd_fabric import paths
from frojd_fabric.hooks import hook
from frojd_fabric.ext import composer


@hook("init_tasks")
def init_tasks():
    # Remove trailing slash
    if "public_path" in env:
        public_path = env.public_path.rstrip("/")
        env.public_path = public_path


@hook("setup")
def setup():
    env.run("touch %s" % paths.get_shared_path("wp-config.php"))
    env.run("chmod 400 %s" % paths.get_shared_path("wp-config.php"))

    env.run("touch %s" % paths.get_shared_path(".htaccess"))
    env.run("chmod 644 %s" % paths.get_shared_path(".htaccess"))


@hook("deploy")
def after_deploy():
    paths.symlink(
        paths.get_shared_path(".htaccess"),
        paths.get_current_path(".htaccess")
    )

    paths.symlink(
        paths.get_shared_path("wp-config.php"),
        paths.get_current_path("wp-config.php")
    )

    paths.symlink(
        paths.get_upload_path(),
        paths.get_current_path("wp-content/uploads")
    )

    if "public_path" in env:
        paths.symlink(paths.get_current_path(), env.public_path)




# -*- coding: utf-8 -*-

"""
frojd_fabric.recipes.django
---------------------------
This recipe holds the most standard components of a django installation.
"""

from fabric.state import env
from fabric.context_managers import prefix
from frojd_fabric.ext import envfile, virtualenv
from frojd_fabric import paths
from frojd_fabric.hooks import hook


@hook("init_tasks")
def init_tasks():
    # Remove trailing slash
    if "public_path" in env:
        public_path = env.public_path.rstrip("/")
        env.public_path = public_path


@hook("setup")
def setup():
    envfile.create_env()
    virtualenv.create_venv()


@hook("deploy")
def after_deploy():
    envfile.symlink_env()

    with prefix("source %s" % (virtualenv.get_path()+"/bin/activate")):
        virtualenv.update_requirements()
        _migrate()

    if "public_path" in env:
        paths.symlink(paths.get_current_path(), env.public_path)


def _migrate():
    with(env.cd(paths.get_current_path())):
        env.run("python manage.py migrate")


@hook("rollback")
def rollback():
    # TODO: Add migration rollback logic.
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
fabrik
------------

:copyright: (c) 2014-2015 by Fröjd Interactive AB
:license: MIT, see LICENSE for more details.
"""


__title__ = "fabrik"
__version__ = "1.2.2"
__build__ = 122
__license__ = "MIT"
__copyright__ = "Copyright 2014-2015 Fröjd Interactive AB"



# -*- coding: utf-8 -*-

"""
frojd_fabric.recipes.node
---------------------------------
Recipe for node.js that includes envfile
"""

from fabric.state import env
from frojd_fabric.hooks import hook
from frojd_fabric.ext import npm, forever, envfile
from frojd_fabric import paths


@hook("setup")
def setup():
    envfile.create_env()


@hook("deploy")
def deploy():
    envfile.symlink_env()

    with(env.cd(paths.get_current_path())):
        npm.install()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import pip

from setuptools import setup, find_packages
from pip.req import parse_requirements
import frojd_fabric


if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()

package_exclude = ("tests*", "examples*")
packages = find_packages(exclude=package_exclude)

# Handle requirements
requires = parse_requirements("requirements/install.txt",
                              session=pip.download.PipSession())
install_requires = [str(ir.req) for ir in requires]

requires = parse_requirements("requirements/tests.txt",
                              session=pip.download.PipSession())
tests_require = [str(ir.req) for ir in requires]


# Convert markdown to rst
try:
    from pypandoc import convert
    long_description = convert("README.md", "rst")
except:
    long_description = ""


setup(
    name="fabrik",
    version=frojd_fabric.__version__,
    description=("A simple to use deployment toolkit built on top of Fabric"),
    long_description=long_description,
    author="Fröjd",
    author_email="martin.sandstrom@frojd.se",
    url="https://github.com/frojd/fabrik",
    packages=packages,
    include_package_data=True,
    install_requires=install_requires,
    tests_require=tests_require,
    entry_points={
        "console_scripts": [
            "fabrik = frojd_fabric.scripts.fabrik:main",
        ]
    },
    license="MIT",
    zip_safe=False,
    classifiers=(
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Build Tools",
        "Topic :: Software Development :: Libraries",
        "Topic :: System :: Software Distribution",
        "Topic :: System :: Systems Administration",
    ),
)



# -*- coding: utf-8 -*-

"""
frojd_fabric.ext.gunicorn
-------------------------
Contains methods for dealing with the gunicorn server.
"""

from fabric.decorators import task
from fabric.state import env


@task
def restart_gunicorn():
    env.run("service gunicorn restart")



# -*- coding: utf-8 -*-

"""
frojd_fabric.ext.nginx
----------------------
Methods for handling nginx
"""

from fabric.state import env


def restart():
    env.run("service nginx restart")


def reload():
    env.run("nginx -s reload")

